/* which -- reports the first or every occurrence of FILE in PATH (default
	   Run$Path).
   Copyright (C) 2000,2002 Darren Salt.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:
   1. Redistributions of source code must retain the above copyright notice,
   this list of conditions and the following disclaimer.
   2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
   3. The name of the author may not be used to endorse or promote products
   derived from this software without specific prior written permission.

   This software is provided by the author "as is" and any express or implied
   warranties including, but not limited to, the implied warranties of
   merchantability and fitness for a particular purpose are disclaimed. in no
   event shall the author be liable for any direct, indirect, incidental,
   special, exemplary, or consequential damages (including, but not limited
   to, procurement of substitute goods or services; loss of use, data, or
   profits; or business interruption) however caused and on any theory of
   liability, whether in contract, strict liability, or tort (including
   negligence or otherwise) arising in any way out of the use of this
   software, even if advised of the possibility of such damage.
 */

/* Written by Darren Salt <ds@youmustbejoking.demon.co.uk>.  */

#ifndef __riscos
#error This program only runs on RISC OS
#endif

#define VERSION "0.2.1"

#include "system.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <kernel.h>

#include "getopt.h"
#include "ro_wild.h"
#include "alloca.h"
#include "error.h"

const char *program_name;


/* Display help text then exit. */
static void
help (void)
{
  printf ("\
Usage: %s [OPTIONS] FILESPEC...\n\
Find the first or every match for FILESPEC in Run: or the specified path(s).\n\
\n\
  -a, --all	   find every match\n\
  -s, --alias	   look for a command alias (all if -a)\n\
  -p, --path=PATH  scan this path instead of Run: [cumulative]\n\
  -h, --help	   display this help text\n\
  -v, --version	   display version and licence information\n\
\n\
Bug reports to <ds@youmustbejoking.demon.co.uk>.\n\
", program_name);
  exit (0);
}


static void
version (void)
{
  puts ("\
which " VERSION "\n\
\n\
Copyright (C) 2000,2002 Darren Salt.\n\
This is free software; see the source for copying conditions. There is NO\n\
warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\
");
  exit (0);
}


static const char short_options[] = "asp:hv";

static struct option long_options[] = {
  {"all", no_argument, NULL, 'a'},
  {"alias", no_argument, NULL, 's'},
  {"help", no_argument, NULL, 'h'},
  {"version", no_argument, NULL, 'v'},
  {NULL, 0, NULL, 0}
};


static int
scan (const char *filespec, const char *path, int all)
{
  ro_wild *wild;
  int l = strlen (path);
  char *afsp = alloca (l + strlen (filespec) + 2);
  char *file;

  if (!afsp)
    error (2, ENOMEM, "out of memory");
  strcpy (afsp, path);
  if (afsp[l - 1] != '.' && afsp[l - 1] != ':')
    afsp[l++] = '.';
  strcpy (afsp + l, filespec);

  wild = ro_wild_init (afsp);
  while ((file = ro_wild_next (wild)) != 0)
    {
      _swi (8, 3 | 1 << 31, 17, file, &l);
      if (l == 1)
	{			/* normal files only */
	  static char *f = 0;
	  _swi (0x29 /*OS_FSControl */ , 0x3F | 1 << (31 - 5),
		37, file, 0, 0, 0, 0, &l);
	  f = xrealloc (f, l = 1 - l);
	  _swi (0x29 /*OS_FSControl */ , 0x3F,
		37, file, f, 0, 0, l);
	  puts (f);
	  if (!all)
	    {
	      ro_wild_finish (wild);
	      return 1;
	    }
	}
    }
  ro_wild_finish (wild);
  return 0;
}


int
main (int argc, char *argv[])
{
  int all = 0, alias = 0, c;
  const char **path = 0;
  int num_paths = 0;

  program_name = argv[0];

  while ((c = getopt_long (argc, argv, short_options, long_options, (int *) 0))
	 != EOF)
    switch (c)
      {
      case 'a':
	all = 1;
	break;
      case 'p':
	path = xrealloc (path, ++num_paths * sizeof (char *));
	path[num_paths - 1] = xstrdup (optarg);
	break;
      case 's':
	alias = 1;
	break;
      case 'h':
	help ();
      case 'v':
	version ();
      }

  if (optind >= argc)
    error (1, 0, "missing file specifier");

  if (num_paths == 0)
    {
      path = xmalloc (sizeof (char *));
      path[0] = "Run:";
      num_paths = 1;
    }

  optind--;
  while (++optind < argc)
    {
      int i = 0;
      if (alias)
	{
	  char *var = xmalloc (7 + strlen (argv[optind]));
	  sprintf (var, "Alias$%s", argv[optind]);
	  do
	    {
	      int length = 0;
	      checkvar (var, i, &length, &i);
//	      _swix (0x23 /* OS_ReadVarVal */ , 0x1F | 3 << 28,
//		     var, 0, -1, i, 0, &length, &i);
	      if (length < 0)
		puts ((const char *) var);
	    }
	  while (i && all);
	  free (var);
	}
      for (i = 0; i < num_paths; ++i)
	if (scan (argv[optind], path[i], all) && !all)
	  return 0;
    }

  return 0;
}
